@@ -1,6 +1,8 @@
-//! This crate defines a buffer data structure optimized to be written to and
-//! read from standard `Vec`s. `DataBuffer` is particularly useful when dealing
-//! with data whose type is determined at run time.
+//! This crate defines a buffer data structure optimized to be written to and read from standard
+//! `Vec`s. `DataBuffer` is particularly useful when dealing with data whose type is determined at
+//! run time.  Note that data is stored in the underlying byte buffers in native endian form, thus
+//! requesting typed data from a buffer on a platform with different endianness will produce
+//! undefined behavior.
 
 pub extern crate reinterpret;
 
@@ -34,7 +36,7 @@ pub struct DataBuffer {
 }
 
 impl DataBuffer {
-    /// Construct an empty `DataBuffer`
+    /// Construct an empty `DataBuffer`.
     #[inline]
     pub fn new() -> Self {
         DataBuffer {
@@ -44,6 +46,47 @@ impl DataBuffer {
         }
     }
 
+    /// Construct an empty `DataBuffer` with a specific type.
+    #[inline]
+    pub fn with_type<T: Any>() -> Self {
+        DataBuffer {
+            data: Vec::new(),
+            length: 0,
+            type_id: TypeId::of::<T>(),
+        }
+    }
+
+    /// Construct an empty `DataBuffer` with a specific type given by the type id.
+    #[inline]
+    pub fn with_type_id(id: TypeId) -> Self {
+        DataBuffer {
+            data: Vec::new(),
+            length: 0,
+            type_id: id,
+        }
+    }
+
+    /// Construct an empty `DataBuffer` with a capacity for a given number of typed elements. For
+    /// setting byte capacity use `with_byte_capacity`.
+    #[inline]
+    pub fn with_capacity<T: Any>(n: usize) -> Self {
+        DataBuffer {
+            data: Vec::with_capacity(n*size_of::<T>()),
+            length: 0,
+            type_id: TypeId::of::<T>(),
+        }
+    }
+
+    /// Construct an empty `DataBuffer` with a capacity for a given number of bytes.
+    #[inline]
+    pub fn with_byte_capacity(n: usize) -> Self {
+        DataBuffer {
+            data: Vec::with_capacity(n),
+            length: 0,
+            type_id: TypeId::of::<()>(),
+        }
+    }
+
     /// Construct a typed `DataBuffer` with a given size and filled with the specified default
     /// value.
     /// #  Examples
@@ -148,6 +191,20 @@ impl DataBuffer {
         Some(self)
     }
 
+    /// Add an element to this buffer. If the type of the given element coincides with the type
+    /// stored by this buffer, then the modified buffer is returned via a mutable reference.
+    /// Otherwise, `None` is returned.
+    #[inline]
+    pub fn push<T: Any + std::fmt::Debug>(&mut self, element: T) -> Option<&mut Self> {
+        self.check_ref::<T>()?;
+        let element_ref = &element;
+        let element_byte_ptr = element_ref as *const T as *const u8;
+        let element_byte_slice = unsafe {
+            slice::from_raw_parts(element_byte_ptr, size_of::<T>())
+        };
+        self.push_bytes(element_byte_slice)
+    }
+
     /// Check if the current buffer contains elements of the specified type. Returns `Some(self)`
     /// if the type matches and `None` otherwise.
     #[inline]
@@ -185,6 +242,12 @@ impl DataBuffer {
         self.length
     }
 
+    /// Get the byte capacity of this buffer.
+    #[inline]
+    pub fn byte_capacity(&self) -> usize {
+        self.data.capacity()
+    }
+
     /// Get the size of the element type.
     pub fn size_of_type(&self) -> usize {
         // Check that the length of the buffer is a multiple of the interpreted vector.
@@ -219,25 +282,6 @@ impl DataBuffer {
         self.as_mut_slice::<T>().map(|x| x.iter_mut())
     }
 
-    /// Iterate over chunks type sized chunks of bytes without interpreting them. This avoids
-    /// needing to know what type data you're dealing with. This type of iterator is useful for
-    /// transferring data from one place to another for a generic buffer.
-    #[inline]
-    pub fn byte_chunks<'a>(&'a self) -> impl Iterator<Item=&'a [u8]> + 'a {
-        let chunk_size = self.size_of_type();
-        self.data.chunks(chunk_size)
-    }
-
-    /// Mutably iterate over chunks type sized chunks of bytes without interpreting them. This
-    /// avoids needing to know what type data you're dealing with. This type of iterator is useful
-    /// for transferring data from one place to another for a generic buffer, or modifying the
-    /// underlying untyped bytes (e.g. bit twiddling).
-    #[inline]
-    pub fn byte_chunks_mut<'a>(&'a mut self) -> impl Iterator<Item=&'a mut [u8]> + 'a {
-        let chunk_size = self.size_of_type();
-        self.data.chunks_mut(chunk_size)
-    }
-
     /// Append cloned items from this buffer to a given `Vec<T>`. Return the mutable reference
     /// `Some(vec)` if type matched the internal type and `None` otherwise.
     #[inline]
@@ -405,6 +449,56 @@ impl DataBuffer {
         self.data.as_mut_slice()
     }
 
+    /// Iterate over chunks type sized chunks of bytes without interpreting them. This avoids
+    /// needing to know what type data you're dealing with. This type of iterator is useful for
+    /// transferring data from one place to another for a generic buffer.
+    #[inline]
+    pub fn byte_chunks<'a>(&'a self) -> impl Iterator<Item=&'a [u8]> + 'a {
+        let chunk_size = self.size_of_type();
+        self.data.chunks(chunk_size)
+    }
+
+    /// Mutably iterate over chunks type sized chunks of bytes without interpreting them. This
+    /// avoids needing to know what type data you're dealing with. This type of iterator is useful
+    /// for transferring data from one place to another for a generic buffer, or modifying the
+    /// underlying untyped bytes (e.g. bit twiddling).
+    #[inline]
+    pub fn byte_chunks_mut<'a>(&'a mut self) -> impl Iterator<Item=&'a mut [u8]> + 'a {
+        let chunk_size = self.size_of_type();
+        self.data.chunks_mut(chunk_size)
+    }
+
+    /// Add bytes to this buffer. If the size of the given slice coincides with the number of bytes
+    /// occupied by the underlying element type, then these bytes are added to the underlying data
+    /// buffer and a mutable reference to the buffer is returned.
+    /// Otherwise, `None` is returned, and the buffer remains unmodified.
+    #[inline]
+    pub fn push_bytes(&mut self, bytes: &[u8]) -> Option<&mut Self> {
+        if bytes.len() == self.size_of_type() { 
+            self.data.extend_from_slice(bytes);
+            self.length += 1;
+            Some(self)
+        } else {
+            None
+        }
+    }
+
+    /// Add bytes to this buffer. If the size of the given slice is a multiple of the number of bytes
+    /// occupied by the underlying element type, then these bytes are added to the underlying data
+    /// buffer and a mutable reference to the buffer is returned.
+    /// Otherwise, `None` is returned and the buffer is unmodified.
+    #[inline]
+    pub fn extend_bytes(&mut self, bytes: &[u8]) -> Option<&mut Self> {
+        let size_of_type = self.size_of_type();
+        if bytes.len() % size_of_type == 0 { 
+            self.data.extend_from_slice(bytes);
+            self.length += bytes.len() / size_of_type;
+            Some(self)
+        } else {
+            None
+        }
+    }
+
     /*
      * Methods specific to buffers storing numeric data
      */
@@ -484,6 +578,42 @@ impl fmt::Display for DataBuffer {
 mod tests {
     use super::*;
 
+    /// Test various ways to create a data buffer.
+    #[test]
+    fn initialization_test() {
+        // Empty untyped buffer.
+        let a = DataBuffer::new();
+        assert_eq!(a.len(), 0);
+        assert_eq!(a.bytes_ref().len(), 0);
+        assert_eq!(a.type_id(), TypeId::of::<()>());
+
+        // Empty typed buffer.
+        let a = DataBuffer::with_type::<f32>();
+        assert_eq!(a.len(), 0);
+        assert_eq!(a.bytes_ref().len(), 0);
+        assert_eq!(a.type_id(), TypeId::of::<f32>());
+
+        // Empty buffer typed by the given type id.
+        let a = DataBuffer::with_type_id(TypeId::of::<f64>());
+        assert_eq!(a.len(), 0);
+        assert_eq!(a.bytes_ref().len(), 0);
+        assert_eq!(a.type_id(), TypeId::of::<f64>());
+
+        // Empty typed buffer with a given capacity.
+        let a = DataBuffer::with_capacity::<f32>(4);
+        assert_eq!(a.len(), 0);
+        assert_eq!(a.bytes_ref().len(), 0);
+        assert_eq!(a.byte_capacity(), 4*size_of::<f32>());
+        assert_eq!(a.type_id(), TypeId::of::<f32>());
+
+        // Empty untyped buffer with a given byte capacity.
+        let a = DataBuffer::with_byte_capacity(4);
+        assert_eq!(a.len(), 0);
+        assert_eq!(a.bytes_ref().len(), 0);
+        assert_eq!(a.byte_capacity(), 4);
+        assert_eq!(a.type_id(), TypeId::of::<()>());
+    }
+
     #[test]
     fn data_integrity_u8_test() {
         let vec = vec![1u8, 3, 4, 1, 2];
@@ -713,4 +843,44 @@ mod tests {
             assert_eq!(reinterpret::reinterpret_slice::<u8, f32>(val)[0], vec_f32[i]);
         }
     }
+
+    /// Test pushing values and bytes to a buffer.
+    #[test]
+    fn push_test() {
+        let mut vec_f32 = vec![1.0_f32, 23.0, 0.01];
+        let mut buf = DataBuffer::from(vec_f32.clone()); // Convert into buffer
+        for (i, &val) in buf.iter::<f32>().unwrap().enumerate() {
+            assert_eq!(val, vec_f32[i]);
+        }
+
+        vec_f32.push(42.0f32);
+        buf.push(42.0f32).unwrap(); // must provide explicit type
+
+        for (i, &val) in buf.iter::<f32>().unwrap().enumerate() {
+            assert_eq!(val, vec_f32[i]);
+        }
+
+        vec_f32.push(11.43);
+        buf.push(11.43f32).unwrap();
+
+        for (i, &val) in buf.iter::<f32>().unwrap().enumerate() {
+            assert_eq!(val, vec_f32[i]);
+        }
+
+        // Zero float is always represented by four zero bytes in IEEE format.
+        vec_f32.push(0.0);
+        vec_f32.push(0.0);
+        buf.extend_bytes(&[0,0,0,0, 0,0,0,0]).unwrap();
+
+        for (i, &val) in buf.iter::<f32>().unwrap().enumerate() {
+            assert_eq!(val, vec_f32[i]);
+        }
+
+        vec_f32.push(0.0);
+        buf.push_bytes(&[0,0,0,0]).unwrap();
+
+        for (i, &val) in buf.iter::<f32>().unwrap().enumerate() {
+            assert_eq!(val, vec_f32[i]);
+        }
+    }
 }
